@@ -1,4 +1,3 @@
-# coding=utf-8
 """
 HTTP Client Library Adapters
 
@@ -12,10 +11,35 @@ from hvac import utils
 from hvac.constants.client import DEFAULT_URL
 
 
-class Adapter(object):
+class Adapter(metaclass=ABCMeta):
     """Abstract base class used when constructing adapters for use with the Client class."""
 
-    __metaclass__ = ABCMeta
+    @classmethod
+    def from_adapter(
+        cls,
+        adapter,
+    ):
+        """Create a new adapter based on an existing Adapter instance.
+        This can be used to create a new type of adapter that inherits the properties of an existing one.
+
+        :param adapter: The existing Adapter instance.
+        :type adapter: hvac.Adapters.Adapter
+        """
+
+        return cls(
+            base_uri=adapter.base_uri,
+            token=adapter.token,
+            cert=adapter._kwargs.get("cert"),
+            verify=adapter._kwargs.get("verify"),
+            timeout=adapter._kwargs.get("timeout"),
+            proxies=adapter._kwargs.get("proxies"),
+            allow_redirects=adapter.allow_redirects,
+            session=adapter.session,
+            namespace=adapter.namespace,
+            ignore_exceptions=adapter.ignore_exceptions,
+            strict_http=adapter.strict_http,
+            request_header=adapter.request_header,
+        )
 
     def __init__(
         self,
@@ -30,6 +54,7 @@ class Adapter(object):
         namespace=None,
         ignore_exceptions=False,
         strict_http=False,
+        request_header=True,
     ):
         """Create a new request adapter instance.
 
@@ -59,9 +84,21 @@ class Adapter(object):
         :type ignore_exceptions: bool
         :param strict_http: If True, use only standard HTTP verbs in request with additional params, otherwise process as is
         :type strict_http: bool
+        :param request_header: If true, add the X-Vault-Request header to all requests to protect against SSRF vulnerabilities.
+        :type request_header: bool
         """
         if not session:
             session = requests.Session()
+            session.cert, session.verify, session.proxies = cert, verify, proxies
+        # fix for issue 991 using session verify if set
+        else:
+            if session.verify:
+                # need to set the variable and not assign it to self so it is properly passed in kwargs
+                verify = session.verify
+            if session.cert:
+                cert = session.cert
+            if session.proxies:
+                proxies = session.proxies
 
         self.base_uri = base_uri
         self.token = token
@@ -70,6 +107,7 @@ class Adapter(object):
         self.allow_redirects = allow_redirects
         self.ignore_exceptions = ignore_exceptions
         self.strict_http = strict_http
+        self.request_header = request_header
 
         self._kwargs = {
             "cert": cert,
@@ -176,13 +214,13 @@ class Adapter(object):
         """Perform a login request.
 
         Associated request is typically to a path prefixed with "/v1/auth") and optionally stores the client token sent
-            in the resulting Vault response for use by the :py:meth:`hvac.adapters.Adapter` instance under the _adapater
+            in the resulting Vault response for use by the :py:meth:`hvac.adapters.Adapter` instance under the _adapter
             Client attribute.
 
         :param url: Path to send the authentication request to.
         :type url: str | unicode
         :param use_token: if True, uses the token in the response received from the auth request to set the "token"
-            attribute on the the :py:meth:`hvac.adapters.Adapter` instance under the _adapater Client attribute.
+            attribute on the the :py:meth:`hvac.adapters.Adapter` instance under the _adapter Client attribute.
         :type use_token: bool
         :param kwargs: Additional keyword arguments to include in the params sent with the request.
         :type kwargs: dict
@@ -206,13 +244,6 @@ class Adapter(object):
         """
         return NotImplementedError
 
-    @utils.deprecated_method(
-        to_be_removed_in_version="0.9.0",
-        new_method=login,
-    )
-    def auth(self, url, use_token=True, **kwargs):
-        return self.login(url=url, use_token=use_token, **kwargs)
-
     @abstractmethod
     def request(self, method, url, headers=None, raise_exception=True, **kwargs):
         """Main method for routing HTTP requests to the configured Vault base_uri. Intended to be implement by subclasses.
@@ -242,6 +273,34 @@ class RawAdapter(Adapter):
     but always returns Response objects for requests.
     """
 
+    def _raise_for_error(self, method: str, url: str, response: requests.Response):
+        msg = json = text = errors = None
+        try:
+            text = response.text
+        except Exception:
+            pass
+
+        if response.headers.get("Content-Type") == "application/json":
+            try:
+                json = response.json()
+            except Exception:
+                pass
+            else:
+                errors = json.get("errors")
+
+        if errors is None:
+            msg = text
+
+        utils.raise_for_error(
+            method,
+            url,
+            response.status_code,
+            msg,
+            errors=errors,
+            text=text,
+            json=json,
+        )
+
     def get_login_token(self, response):
         """Extracts the client token from a login response.
 
@@ -281,6 +340,9 @@ class RawAdapter(Adapter):
         if not headers:
             headers = {}
 
+        if self.request_header:
+            headers["X-Vault-Request"] = "true"
+
         if self.token:
             headers["X-Vault-Token"] = self.token
 
@@ -311,17 +373,7 @@ class RawAdapter(Adapter):
         )
 
         if not response.ok and (raise_exception and not self.ignore_exceptions):
-            text = errors = None
-            if response.headers.get("Content-Type") == "application/json":
-                try:
-                    errors = response.json().get("errors")
-                except Exception:
-                    pass
-            if errors is None:
-                text = response.text
-            utils.raise_for_error(
-                method, url, response.status_code, text, errors=errors
-            )
+            self._raise_for_error(method, url, response)
 
         return response
 
@@ -353,7 +405,7 @@ class JSONAdapter(RawAdapter):
         :return: Dict on HTTP 200 with JSON body, otherwise the response object.
         :rtype: dict | requests.Response
         """
-        response = super(JSONAdapter, self).request(*args, **kwargs)
+        response = super().request(*args, **kwargs)
         if response.status_code == 200:
             try:
                 return response.json()
